%module svmc
%{
#include "svm.h"
%}

enum { C_SVC, C_SVCq, R2, R2q, NU_SVR };	/* svm_type */
enum { LINEAR, POLY, RBF, SIGMOID };	/* kernel_type */

%pragma make_default

struct svm_parameter
{
	int svm_type;
	int kernel_type;
	double degree;	// for poly
	double gamma;	// for poly/rbf/sigmoid
	double coef0;	// for poly/sigmoid

	// these are for training only
	double cache_size; // in MB
	double eps;	// stopping criteria
	double C;	// for C_SVC, EPSILON_SVR and NU_SVR
	int nr_weight;		// for C_SVC
	int *weight_label;	// for C_SVC
	double* weight;		// for C_SVC
	double nu;	// for NU_SVC, ONE_CLASS, and NU_SVR
	double p;	// for EPSILON_SVR
	int shrinking;	// use the shrinking heuristics
	int cal_partial;	// calculate the partial derivatives
};

struct svm_problem
{
	int l;
	double *y;
	struct svm_node **x;
};

struct svm_model
{
	struct svm_parameter param;	// parameter
	int nr_class;		// number of classes, = 2 in regression/one class svm
	int l;			// total #SV
	struct svm_node **SV;		// SVs (SV[l])
	double **sv_coef;	// coefficients for SVs in decision functions (sv_coef[n-1][l])
	double *rho;		// constants in decision functions (rho[n*(n-1)/2])
	double obj, wsq, cxi, rsq;	// should be arrays for multiclass, but just values for now

	// for classification only

	int *label;		// label of each class (label[n])
	int *nSV;		// number of SVs for each class (nSV[n])
				// nSV[0] + nSV[1] + ... + nSV[n-1] = l
	int *nBSV;
	// XXX
	int free_sv;		// 1 if svm_model is created by svm_load_model
				// 0 if svm_model is created by svm_train
//	double partial;		// 1/2 \partial\|w\|^2 / \partial\sigma^2 or \partial R^2 / \partial C
};

%pragma no_default

struct svm_model *svm_train(const struct svm_problem *prob,
			    const struct svm_parameter *param);

int svm_save_model(const char *model_file_name, const struct svm_model *model);

struct svm_model *svm_load_model(const char *model_file_name);

double svm_predict(const struct svm_model *model, const struct svm_node *x);

void svm_destroy_model(struct svm_model *model);

double svm_calculate_partial(const struct svm_model *model);

%include array.i		// for double array operations

%inline %{
static double matrix_double_get(double **double_matrix, int index1, int index2)
{
    if (double_matrix)
        if (double_matrix[index1])
            return double_matrix[index1][index2];
    return DBL_MIN;
}

struct svm_node *svm_node_array(int size)
{
	return (struct svm_node *)malloc(sizeof(struct svm_node)*size);
}

void svm_node_array_set(struct svm_node *array, int i, int index, double value)
{
	array[i].index = index;
	array[i].value = value;
}

void svm_node_array_destroy(struct svm_node *array)
{
	free(array);
}

struct svm_node **svm_node_matrix(int size)
{
	return (struct svm_node **)malloc(sizeof(struct svm_node *)*size);	
}

void svm_node_matrix_set(struct svm_node **matrix, int i, struct svm_node* array)
{
	matrix[i] = array;
}

void svm_node_matrix_destroy(struct svm_node **matrix)
{
	free(matrix);
}

int matrix_svm_node_index_get(struct svm_node **matrix, int index1, int index2)
{
    if (matrix)
        if (matrix[index1])
            return matrix[index1][index2].index;
    return 0;
}

double matrix_svm_node_value_get(struct svm_node **matrix, int index1, int index2)
{
    if (matrix)
        if (matrix[index1])
            return matrix[index1][index2].value;
    return DBL_MIN;
}
%}
